/*!
  @file adxl_util.h
  @brief Common routines for ADXL board - open,close, read, sighandler
  @date 5/3/03
  @author Paul Hubbard
*/
/*!
  @mainpage
  @section intro Introduction

  This code reads from a Crossbow ADXL202EVB board. This is a small
  evaluation board for the Analog Devices ADXL202 2-axis accelerometer.

  @section apps Applications

  You get two applications from this - adxl-simple, which prints one
  reading per second, and nees-adxl, which reads and streams data out
  to the NEESgrid driver / NSDS.

  @section deps Dependencies

  This code uses two other neesgrid packages, 'flog' and 'nsds-driver' for
  their common subroutines. Code re-use; it's not just a theory. You have
  to build those before this can be compiled. This assumes they are in
  ../flog and ../nsds-driver

  @section other Other Stuff

  Some of this code is cribbed from the net; also included are xbow.m (a Matlab
  interface) and xbow.cpp, a simple C++ app to read and print. 
*/

#if !defined(_ADXL_UTIL_H_)
#define _ADXL_UTIL_H_

#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include <stdint.h>
#include <string.h>
#include <signal.h>
#include <time.h>

#include <sys/termios.h>
#include <fcntl.h>

extern bool control_break;


//! Number of possible data channels - analog plus digital
#define NUM_ADXL_CHANNELS 2

//! Number of bytes per ADXL reading
#define ADXL_READ_LEN 4

//! Size of a NSDS-formatted reading - used for buffer sizing.
#define ADXL_DATUM_LEN 24

//! Max sample rate, approximate
#define ADXL_MAX_RATE_HZ 250

//! Struct used to return raw and processed readings from the ADXL
typedef struct
{
    int16_t       raw[NUM_ADXL_CHANNELS];
    double        G[NUM_ADXL_CHANNELS];
} adxl_reading_t;

// Function prototypes
int adxl_open(const char *comport_file);
int adxl_read(const int comm_fd, adxl_reading_t *result);
int adxl_close(const int comm_fd);
void adxl_sighandler(int signal);

#endif
